"{ Package: 'jn:refactoring_custom' }"

CustomCodeGenerator subclass:#CustomCodeGeneratorClassGenerator
	instanceVariableNames:''
	classVariableNames:''
	poolDictionaries:''
	category:'Interface-Refactoring-Custom-Generators'
!


!CustomCodeGeneratorClassGenerator class methodsFor:'accessing-presentation'!

description
    "Returns more detailed description of the receiver"

    ^ 'Generates template class for code generator'

    "Modified: / 29-03-2014 / 19:38:48 / Jakub Nesveda <nesvejak@fit.cvut.cz>"
!

group
    "Returns a collection strings describing a group to which
     receiver belongs. A groups may be nested hence the array of
     strings. For example for subgroup 'Accessors' in group 'Generators'
     this method should return #('Generators' 'Accessors')."

    "/ By default return an empty array which means the item will appear
    "/ in top-level group.
    ^ #(Generators)

    "Created: / 22-08-2014 / 18:48:04 / Jan Vrany <jan.vrany@fit.cvut.cz>"
!

label
    "Returns show label describing the receiver. This label
     is used in UI as menu item/tree item label."

    ^ 'New Code Generator'

    "Modified: / 29-03-2014 / 19:21:56 / Jakub Nesveda <nesvejak@fit.cvut.cz>"
    "Modified: / 22-08-2014 / 18:50:45 / Jan Vrany <jan.vrany@fit.cvut.cz>"
! !

!CustomCodeGeneratorClassGenerator class methodsFor:'queries'!

availableInContext: aCustomContext
    "Returns true if the generator/refactoring is available in given
     context, false otherwise. 

     Called by the UI to figure out what generators / refactorings 
     are available at given point. See class CustomContext for details."

    ^ true
!

availableInPerspective:aCustomPerspective 
    "Returns true if the generator/refactoring is available in given
     perspective, false otherwise.

     Called by the UI to figure out what generators / refactorings
     to show"

    ^ aCustomPerspective isClassPerspective
! !

!CustomCodeGeneratorClassGenerator methodsFor:'code generation'!

buildForClassName: aClassName 

    | subContext generator class |

    class := codeBuilder createClass.
    class
        superclassName: 'CustomCodeGenerator';
        className: aClassName;
        category: self class category.

    subContext := CustomSubContext new.
    subContext selectedClasses: (Array with: class).

    generator := CustomSubclassResponsibilityCodeGenerator subGeneratorOf: self.  
    generator executeInContext: subContext

    "Created: / 24-04-2014 / 10:15:22 / Jakub Nesveda <nesvejak@fit.cvut.cz>"
    "Modified: / 26-04-2014 / 16:23:02 / Jakub Nesveda <nesvejak@fit.cvut.cz>"
    "Modified: / 05-08-2014 / 13:37:16 / Jan Vrany <jan.vrany@fit.cvut.cz>"
! !

!CustomCodeGeneratorClassGenerator methodsFor:'executing'!

buildInContext: aCustomContext 

    | generatorClassName |

    generatorClassName := dialog 
                            requestClassName: 'Enter class name for new generator' 
                            initialAnswer: 'CustomXXXCodeGenerator'.

    self buildForClassName: generatorClassName

    "Created: / 31-03-2014 / 18:07:56 / Jakub Nesveda <nesvejak@fit.cvut.cz>"
    "Modified: / 03-04-2014 / 11:42:30 / Jan Vrany <jan.vrany@fit.cvut.cz>"
    "Modified: / 10-05-2014 / 15:39:32 / Jakub Nesveda <nesvejak@fit.cvut.cz>"
! !

!CustomCodeGeneratorClassGenerator class methodsFor:'documentation'!

version_HG

    ^ '$Changeset: <not expanded> $'
! !

