"{ Package: 'jn:refactoring_custom' }"

Object subclass:#CustomCodeGeneratorOrRefactoringBase
	instanceVariableNames:'compositeChangeCollector compositeChangeNesting userPreferences
		confirmChanges generateComments'
	classVariableNames:''
	poolDictionaries:''
	category:'Interface-Refactoring-Custom'
!

!CustomCodeGeneratorOrRefactoringBase class methodsFor:'documentation'!

documentation
"
   A base superclass that provides only undo/redo support. The code has been
   extracted from CodeGeneratorTool.

    [author:]
        Jan Vrany <jan.vrany@fit.cvut.cz>

    [instance variables:]

    [class variables:]

    [see also:]

"
! !

!CustomCodeGeneratorOrRefactoringBase class methodsFor:'compilation'!

compile:theCode forClass:aClass inCategory:cat 
    "install some code for a class.
     If refactory browser stuff is avaliable the refactory tools are used to support undo"

    ^ self new compile:theCode forClass:aClass inCategory:cat
! !

!CustomCodeGeneratorOrRefactoringBase class methodsFor:'private'!

canUseRefactoringSupport
    "check if refactory browser stuff is avaliable"

     ^ RefactoryChangeManager notNil 
    and:[RefactoryChangeManager isLoaded
    and:[UserPreferences current useRefactoringSupport]]
! !

!CustomCodeGeneratorOrRefactoringBase class methodsFor:'testing'!

isAbstract
    ^ self == AbstractCodeGeneratorTool
! !

!CustomCodeGeneratorOrRefactoringBase methodsFor:'bulk changes'!

addChange:aChange
    compositeChangeCollector addChange:aChange
!

executeCollectedChangesNamed:name
    "if both the compositeChangeCollector and I myself think, that it should be confirmed,
     the let user do it - I am not sure, if the heuristics here is useful;
     it might be better to let the generator decide (for example, if it does high-impact
     changes, as opposed to simple ones)"

    compositeChangeCollector notNil ifTrue:[
        compositeChangeNesting := compositeChangeNesting - 1.
        compositeChangeNesting == 0 ifTrue:[
            compositeChangeCollector name:name.
            compositeChangeCollector changesSize == 0 ifTrue:[
                self information:'Nothing generated.'.
            ] ifFalse:[
                "JV@2012-04-04: Here used to be a code for confirming changes.
                 The has moved to RefactoryChangeManager so all refactorings
                 are also subject to confirmation (is enabled in UserPreferences)"
                RefactoryChangeManager instance performChange: compositeChangeCollector
            ].
            compositeChangeCollector := nil.
        ]
    ]

    "Modified (comment): / 04-08-2011 / 17:30:27 / cg"
    "Modified (format): / 04-04-2012 / 16:35:37 / Jan Vrany <jan.vrany@fit.cvut.cz>"
!

startCollectChanges
    (self canUseRefactoringSupport) ifTrue:[
        compositeChangeCollector isNil ifTrue:[
            compositeChangeCollector := CompositeRefactoryChange new.
            compositeChangeNesting := 0.
        ].
        compositeChangeNesting := compositeChangeNesting + 1.
    ]
! !

!CustomCodeGeneratorOrRefactoringBase methodsFor:'compilation'!

compile:theCode forClass:aClass inCategory:cat 
    "install some code for a class.
     If refactory browser stuff is avaliable the refactory tools are used to support undo"

    self
        compile:theCode forClass:aClass inCategory:cat 
        skipIfSame:true
!

compile:theCode forClass:aClass inCategory:categoryOrNil skipIfSame:skipIfSame 
    "install some code for a class.
     If refactory browser stuff is avaliable the refactory tools are used to support undo"

    |change compiler selector oldMethod isSame category|

    isSame := false.
    category := categoryOrNil ? (Compiler defaultMethodCategory).

    skipIfSame ifTrue:[
        compiler := aClass compilerClass new.
        compiler parseMethod:theCode in:aClass ignoreErrors:true ignoreWarnings:true.

        selector := compiler selector.
        selector notNil ifTrue:[
            oldMethod := aClass compiledMethodAt:selector.
            isSame := (oldMethod notNil and:[oldMethod source = theCode]).
            isSame ifTrue:[^ self ].
            oldMethod notNil ifTrue:[
                category := categoryOrNil ? (oldMethod category).
            ].
        ].
    ].

    self canUseRefactoringSupport ifFalse:[
        "/ compile immediately
        aClass compile:theCode classified:category.
        ^ self.
    ].

    change := InteractiveAddMethodChange compile:(theCode asString) in:aClass classified:category.

    "/ if collecting, add to changes (to be executed as one change at the end,
    "/ in order to have only one change in the undo-list (instead of many)
    compositeChangeCollector notNil ifTrue:[
        compositeChangeCollector addChange:change
    ] ifFalse:[
        RefactoryChangeManager performChange:change.
    ]

    "Modified: / 21-08-2006 / 18:39:06 / cg"
    "Modified (format): / 21-01-2012 / 10:40:59 / cg"
! !

!CustomCodeGeneratorOrRefactoringBase methodsFor:'initialization'!

confirmChanges
    "if true, let user confirm complicated changes; if false, just do it"

    ^ confirmChanges ? true

    "Created: / 04-08-2011 / 17:31:45 / cg"
!

confirmChanges:aBoolean
    "if true, let user confirm complicated changes; if false, just do it"

    confirmChanges := aBoolean

    "Created: / 04-08-2011 / 17:26:47 / cg"
!

initialize
    userPreferences := UserPreferences current.
    generateComments := userPreferences generateComments.
! !

!CustomCodeGeneratorOrRefactoringBase methodsFor:'private'!

canUseRefactoringSupport
    "check if refactory browser stuff is avaliable"

     ^ self class canUseRefactoringSupport
! !

!CustomCodeGeneratorOrRefactoringBase class methodsFor:'documentation'!

version_HG

    ^ '$Changeset: <not expanded> $'
! !

