"{ Package: 'jn:refactoring_custom' }"

TestCase subclass:#CustomCodeGeneratorOrRefactoringTestCase
	instanceVariableNames:'context codeBuilder refactoryBuilder generatorOrRefactoring
		classes changeManager userPreferences mock testFormatterSettings
		originalFormatterSettings'
	classVariableNames:''
	poolDictionaries:''
	category:'Interface-Refactoring-Custom-Tests'
!

!CustomCodeGeneratorOrRefactoringTestCase methodsFor:'accessing'!

formatterSettings
    "see setUpFormatterSettings"

    ^ testFormatterSettings

    "Created: / 21-08-2014 / 12:21:00 / Jakub Nesveda <nesvejak@fit.cvut.cz>"
!

generatorOrRefactoring
    "Should return an instance of CustomCodeGenerator or CustomRefactoring subclass"
    
    ^ self subclassResponsibility

    "Created: / 27-05-2014 / 19:16:44 / Jakub Nesveda <nesvejak@fit.cvut.cz>"
    "Modified (comment): / 23-08-2014 / 11:38:04 / Jakub Nesveda <nesvejak@fit.cvut.cz>"
! !

!CustomCodeGeneratorOrRefactoringTestCase methodsFor:'asserting'!

assertClassExists: aClassName

    self assert: (Smalltalk classNamed: aClassName asString) isNil not

    "Created: / 15-06-2014 / 16:42:19 / Jakub Nesveda <nesvejak@fit.cvut.cz>"
!

assertClassMethodSource: aSourceCode atSelector: aSelector
    "Assert that source code is same at given selector for first generated class"

    | className class |

    className := classes first classNameBuilded.
    class := Smalltalk classNamed: className.

    ^ self assertClassMethodSource: aSourceCode atSelector: aSelector forClass: class

    "Created: / 11-07-2014 / 20:11:01 / Jakub Nesveda <nesvejak@fit.cvut.cz>"
!

assertClassMethodSource: expectedSource atSelector: aSelector forClass: aClass
    "Assert that source code is same at given selector for given class"

    | actualSource |

    actualSource := aClass theMetaclass sourceCodeAt: aSelector asSymbol.

    ^ self assertSource: expectedSource sameAs: actualSource

    "Created: / 11-07-2014 / 20:09:59 / Jakub Nesveda <nesvejak@fit.cvut.cz>"
!

assertClassNotExists: aClassName

    self assert: (Smalltalk classNamed: aClassName asString) isNil

    "Created: / 15-06-2014 / 16:42:37 / Jakub Nesveda <nesvejak@fit.cvut.cz>"
!

assertMethodSource: aSourceCode atSelector: aSelector
    "Assert that source code is same at given selector for first generated class"

    | className class |

    className := classes first classNameBuilded.
    class := Smalltalk classNamed: className.

    ^ self assertMethodSource: aSourceCode atSelector: aSelector forClass: class

    "Created: / 27-05-2014 / 20:06:17 / Jakub Nesveda <nesvejak@fit.cvut.cz>"
    "Modified: / 30-05-2014 / 22:36:09 / Jakub Nesveda <nesvejak@fit.cvut.cz>"
!

assertMethodSource: expectedSource atSelector: aSelector forClass: aClass
    "Assert that source code is same at given selector for given class"

    | actualSource |

    actualSource := aClass sourceCodeAt: aSelector asSymbol.

    ^ self assertSource: expectedSource sameAs: actualSource

    "Created: / 27-05-2014 / 20:08:32 / Jakub Nesveda <nesvejak@fit.cvut.cz>"
!

assertSource: expectedSource sameAs: actualSource

    self assert: (Change isSource: expectedSource sameSourceAs: actualSource)

    "Created: / 25-05-2014 / 22:18:10 / Jakub Nesveda <nesvejak@fit.cvut.cz>"
    "Modified: / 10-07-2014 / 11:31:19 / Jakub Nesveda <nesvejak@fit.cvut.cz>"
! !

!CustomCodeGeneratorOrRefactoringTestCase methodsFor:'code generation helpers'!

createClass
    "Returns new class change with some name"

    | class |

    class := codeBuilder createClass.
    class className: 'DummyClassForGeneratorTestCase'.
    classes add: class.

    ^ class

    "Created: / 29-05-2014 / 23:22:52 / Jakub Nesveda <nesvejak@fit.cvut.cz>"
! !

!CustomCodeGeneratorOrRefactoringTestCase methodsFor:'context templates'!

classWithInstanceVariable
    | class |

    class := self createClass.
    class instanceVariableNames: 'instanceVariable'.

    context selectedClasses: (Array with: class)

    "Created: / 29-05-2014 / 00:33:40 / Jakub Nesveda <nesvejak@fit.cvut.cz>"
    "Modified: / 29-05-2014 / 23:44:20 / Jakub Nesveda <nesvejak@fit.cvut.cz>"
!

classWithThreeInstanceVariables
    | class |

    class := self createClass.
    class instanceVariableNames: 'instanceVariable_01 instanceVariable_02 instanceVariable_03'.

    context selectedClasses: (Array with: class).
    context selectedVariables: (class instanceVariableNames).

    "Created: / 13-07-2014 / 21:56:12 / Jakub Nesveda <nesvejak@fit.cvut.cz>"
!

classWithTwoInstanceVariables
    | class |

    class := self createClass.
    class instanceVariableNames: 'instanceVariable_01 instanceVariable_02'.

    context selectedClasses: (Array with: class).
    context selectedVariables: (class instanceVariableNames).

    "Created: / 13-07-2014 / 21:45:06 / Jakub Nesveda <nesvejak@fit.cvut.cz>"
! !

!CustomCodeGeneratorOrRefactoringTestCase methodsFor:'executing'!

createContext: contextTemplateName
    "Creates context by given template name
    (produces more readable code than just method call)"

    self perform: contextTemplateName asSymbol

    "Created: / 27-07-2014 / 12:27:08 / Jakub Nesveda <nesvejak@fit.cvut.cz>"
!

executeGeneratorInContext: contextTemplateName
    "Executes generator in context created by given name"

    self perform: contextTemplateName asSymbol.
    generatorOrRefactoring executeInContext: context

    "Created: / 27-05-2014 / 20:03:14 / Jakub Nesveda <nesvejak@fit.cvut.cz>"
    "Modified: / 29-05-2014 / 23:38:08 / Jakub Nesveda <nesvejak@fit.cvut.cz>"
! !

!CustomCodeGeneratorOrRefactoringTestCase methodsFor:'initialization & release'!

defaultUserPreferences

    userPreferences
        generateComments: true;
        generateCommentsForAspectMethods: true;
        generateCommentsForGetters: true;
        generateCommentsForSetters: true

    "Created: / 09-06-2014 / 22:36:59 / Jakub Nesveda <nesvejak@fit.cvut.cz>"
!

initalizeTestFormatterSettings
    "see setUpFormatterSettings"

    testFormatterSettings := IdentityDictionary new.
    testFormatterSettings
        at: #tabIndent put: 4;  
        at: #spaceAroundTemporaries put: false;  
        at: #emptyLineAfterTemporaries put: true;  
        at: #emptyLineAfterMethodComment put: true;
        at: #spaceAfterReturnToken put: true;  
        at: #spaceAfterKeywordSelector put: false;  
        at: #spaceAfterBlockStart put: true;  
        at: #spaceBeforeBlockEnd put: true;  
        at: #cStyleBlocks put: true;  
        at: #blockArgumentsOnNewLine put: false;  
        at: #maxLengthForSingleLineBlocks put: 4;
        at: #periodAfterLastStatementPolicy put: #keep.
    "
    EmptyLineBeforeFinalMethodComment := true.
    SpaceAroundLiteralArrayElements := true.
    STXStyleMethodComments := true.
    "

    "Created: / 21-08-2014 / 11:38:07 / Jakub Nesveda <nesvejak@fit.cvut.cz>"
!

restoreFormatterSettings
    "see setUpFormatterSettings"

    originalFormatterSettings keysAndValuesDo: [ :key :value |
        RBFormatter perform: key asMutator with: value.   
    ].

    "Created: / 20-08-2014 / 22:25:02 / Jakub Nesveda <nesvejak@fit.cvut.cz>"
!

setUp

    classes := OrderedCollection new.
    mock := CustomMock new.

    context := CustomSubContext new.

    changeManager := CustomLocalChangeManager new.
    self setUpBuilders.

    userPreferences := UserPreferences new.
    self defaultUserPreferences.
    generatorOrRefactoring := self generatorOrRefactoring.

    generatorOrRefactoring isNil ifFalse: [
        self setUpGeneratorOrRefactoring: generatorOrRefactoring
    ].

    self initalizeTestFormatterSettings.
    self setUpFormatterSettings

    "Created: / 27-05-2014 / 19:16:44 / Jakub Nesveda <nesvejak@fit.cvut.cz>"
    "Modified: / 23-08-2014 / 16:00:14 / Jakub Nesveda <nesvejak@fit.cvut.cz>"
!

setUpBuilders

    codeBuilder := CustomSourceCodeBuilder new.
    codeBuilder changeManager: changeManager.

    refactoryBuilder := CustomRefactoryBuilder new.
    refactoryBuilder changeManager: changeManager.

    "Created: / 23-08-2014 / 15:57:43 / Jakub Nesveda <nesvejak@fit.cvut.cz>"
!

setUpFormatterSettings
    "Settings for RBFormatter to keep formatting settings independent on system settings.
    Actually this is not perfect solution because of global scope modifications - imagine
    parallel execution."

    originalFormatterSettings := IdentityDictionary new.
    testFormatterSettings keysAndValuesDo: [ :key :value |
        originalFormatterSettings at: key put: (RBFormatter perform: key).
        RBFormatter perform: key asMutator with: value.   
    ].

    "Created: / 20-08-2014 / 22:21:17 / Jakub Nesveda <nesvejak@fit.cvut.cz>"
    "Modified: / 21-08-2014 / 11:38:24 / Jakub Nesveda <nesvejak@fit.cvut.cz>"
!

setUpGeneratorOrRefactoring: aGeneratorOrRefactoring

    aGeneratorOrRefactoring codeBuilder: codeBuilder.
    aGeneratorOrRefactoring refactoryBuilder: refactoryBuilder.
    aGeneratorOrRefactoring changeManager: changeManager.
    aGeneratorOrRefactoring userPreferences: userPreferences

    "Created: / 23-08-2014 / 15:59:58 / Jakub Nesveda <nesvejak@fit.cvut.cz>"
!

tearDown

    self restoreFormatterSettings.
    changeManager redoChanges.
    mock unmockAll

    "Created: / 27-05-2014 / 19:26:30 / Jakub Nesveda <nesvejak@fit.cvut.cz>"
    "Modified: / 23-08-2014 / 15:54:15 / Jakub Nesveda <nesvejak@fit.cvut.cz>"
! !

