"{ Package: 'jn:refactoring_custom' }"

Object subclass:#CustomMock
	instanceVariableNames:'mockedClasses'
	classVariableNames:'MockCount'
	poolDictionaries:''
	category:'Interface-Refactoring-Custom-Tests'
!

!CustomMock class methodsFor:'documentation'!

documentation
"
    Helper for creating mocked classes.

    [author:]
        Jakub Nesveda <nesvejak@fit.cvut.cz>

"
! !

!CustomMock class methodsFor:'instance creation'!

new
    "return an initialized instance"

    MockCount isNil ifTrue: [ 
        MockCount := 0
    ].

    ^ self basicNew initialize.

    "Created: / 15-06-2014 / 19:20:17 / Jakub Nesveda <nesvejak@fit.cvut.cz>"
! !

!CustomMock methodsFor:'accessing'!

variableNameForClassMockedSelectors

    ^ 'MockedClassSelectors'

    "Created: / 11-07-2014 / 10:16:03 / Jakub Nesveda <nesvejak@fit.cvut.cz>"
!

variableNameForInstanceMockedSelectors

    ^ 'MockedSelectors'

    "Created: / 11-07-2014 / 10:15:10 / Jakub Nesveda <nesvejak@fit.cvut.cz>"
! !

!CustomMock methodsFor:'code generation'!

createMockClassOf: aClass mockNumber: aNumber
    "Returns generated mock subclass of given class 
    to support custom return values of overriden methods"

    | builder mockClass className mockClassName |

    className := aClass new className.
    mockClassName := ('CustomMock', aNumber asString, 'Of', className).

    builder := CustomSourceCodeBuilder new.
    mockClass := builder createClass.
    mockClass
        className: mockClassName;
        superclassName: className;
        classVariableNames: 'MockedSelectors MockedClassSelectors';
        category: 'Interface-Refactoring-Custom-Mocks'.

    self createMockMethodWithSingleValue: mockClass builder: builder.
    self createMockMethodWithBlockValue: mockClass builder: builder.

    builder execute.

    ^ Smalltalk classNamed: mockClassName

    "Created: / 15-06-2014 / 19:15:52 / Jakub Nesveda <nesvejak@fit.cvut.cz>"
    "Modified (comment): / 10-07-2014 / 23:00:22 / Jakub Nesveda <nesvejak@fit.cvut.cz>"
!

createMockMethod: aMockClass methodSource: aSource builder: aBuilder mockedSelectorsVariableName: aMockedSelectorsVariableName
    "Generates mock method for given mock class 
    to support custom method value/implementation by some value/block
    (depends on given source)"

    | method |

    method := aBuilder createMethod.
    method
        class: aMockClass;
        protocol: 'mocking';
        source: aSource;
        replace: '`mockedSelectorsVariableName' with: aMockedSelectorsVariableName.

    "Created: / 11-07-2014 / 10:29:32 / Jakub Nesveda <nesvejak@fit.cvut.cz>"
!

createMockMethodWithBlockValue: aMockClass builder: aBuilder
    "Generates mock method for given mock class 
    to support custom method implementation by block value"

    | instVarName classVarName |

    classVarName := self variableNameForClassMockedSelectors.

    self
        createMockMethod: aMockClass theMetaclass
        methodSource: (self blockValueMethodSource: classVarName)
        builder: aBuilder
        mockedSelectorsVariableName: classVarName.

    instVarName := self variableNameForInstanceMockedSelectors.

    self
        createMockMethod: aMockClass theNonMetaclass
        methodSource: (self blockValueMethodSource: instVarName)
        builder: aBuilder
        mockedSelectorsVariableName: instVarName.

    "Created: / 10-07-2014 / 22:59:49 / Jakub Nesveda <nesvejak@fit.cvut.cz>"
    "Modified: / 11-07-2014 / 12:28:54 / Jakub Nesveda <nesvejak@fit.cvut.cz>"
!

createMockMethodWithSingleValue: aMockClass builder: aBuilder
    "Generates mock method for given mock class 
    to support custom return values of overriden methods"

    | instVarName classVarName |

    classVarName := self variableNameForClassMockedSelectors.

    self
        createMockMethod: aMockClass theMetaclass
        methodSource: (self singleValueMethodSource: classVarName)
        builder: aBuilder
        mockedSelectorsVariableName: classVarName.

    instVarName := self variableNameForInstanceMockedSelectors.

    self
        createMockMethod: aMockClass theNonMetaclass
        methodSource: (self singleValueMethodSource: instVarName)
        builder: aBuilder
        mockedSelectorsVariableName: instVarName.

    "Created: / 10-07-2014 / 22:53:29 / Jakub Nesveda <nesvejak@fit.cvut.cz>"
    "Modified: / 11-07-2014 / 12:28:39 / Jakub Nesveda <nesvejak@fit.cvut.cz>"
! !

!CustomMock methodsFor:'code generation-sources'!

blockValueMethodSource: aMockedSelectorsVariableName
    "Returns mock method source code for given variable name
    which holds mocked selectors and its block values. 
    These values are so called blocks which contains executable code."

    ^ 'mockSelector: aSelector withBlockValue: aBlock

    | valueMethod mockSelector argNumber |

    `mockedSelectorsVariableName isNil ifTrue: [ 
        `mockedSelectorsVariableName := Dictionary new
    ].

    `mockedSelectorsVariableName at: aSelector asSymbol put: aBlock.

    valueMethod := ''value''.
    mockSelector := ''''.
    argNumber := 0.
    (aSelector asString subStrings: $:) do: [ :param | 
        | argName |

        argName := ''arg'', argNumber asString.

        (argNumber > 0) ifTrue: [
            mockSelector := mockSelector, '':'', argName, '' '', param.

            (argNumber > 1) ifTrue: [
                valueMethod := valueMethod, '' value:'', argName.
            ]
            ifFalse: [ 
                valueMethod := valueMethod, '':'', argName.
            ].
        ]
        ifFalse: [ 
            mockSelector := param.
        ].

        valueMethod := valueMethod.
        argNumber := argNumber + 1.
    ].

    (CustomSourceCodeBuilder new)
        class: self class;
        protocol: ''mocks'';
        source: (''`@selector

            ^ (`mockedSelectorsVariableName at: `#selector) `@value'');
        replace: ''`@selector'' with: mockSelector asSymbol;
        replace: ''`#selector'' with: ($#, aSelector);
        replace: ''`@value'' with: valueMethod asSymbol;
        replace: ''`mockedSelectorsVariableName'' with: ''' , aMockedSelectorsVariableName , ''';
        createMethod;
        execute.'.

    "Created: / 10-07-2014 / 22:28:27 / Jakub Nesveda <nesvejak@fit.cvut.cz>"
!

singleValueMethodSource: aMockedSelectorsVariableName
    "Returns mock method source code for given variable name
    which holds mocked selectors and its values. 
    These values are constants (within method) like numbers, classes, objects..."

    ^ 'mockSelector: aSelector withReturnValue: aValue

    | classQuery superclassMethod methodDefinitionTemplate |

    `mockedSelectorsVariableName isNil ifTrue: [ 
        `mockedSelectorsVariableName := Dictionary new
    ].

    `mockedSelectorsVariableName at: aSelector asSymbol put: aValue.

    classQuery := CustomClassQuery new.
    superclassMethod := classQuery methodForSuperclassSelector: aSelector class: self class.
    methodDefinitionTemplate := superclassMethod methodDefinitionTemplate asSymbol.

    (CustomSourceCodeBuilder new)
        class: self class;
        protocol: superclassMethod category;
        source: (''`@selector

            ^ `mockedSelectorsVariableName at: `#selector'');
        replace: ''`@selector'' with: methodDefinitionTemplate;
        replace: ''`#selector'' with: ($#, aSelector);
        replace: ''`mockedSelectorsVariableName'' with: ''' , aMockedSelectorsVariableName , ''';
        createMethod;
        execute.'.

    "Created: / 10-07-2014 / 22:23:58 / Jakub Nesveda <nesvejak@fit.cvut.cz>"
! !

!CustomMock methodsFor:'initialization & release'!

initialize

    mockedClasses := OrderedCollection new

    "Created: / 15-06-2014 / 19:19:21 / Jakub Nesveda <nesvejak@fit.cvut.cz>"
! !

!CustomMock methodsFor:'mock creation'!

mockClassOf: aClass
    "Creates a mock class of given class"
    | mockClass |

    mockClass := self createMockClassOf: aClass mockNumber: MockCount.

    MockCount += 1.
    mockedClasses add: mockClass.

    ^ mockClass

    "Created: / 15-06-2014 / 23:41:59 / Jakub Nesveda <nesvejak@fit.cvut.cz>"
!

mockOf: aClass
    "Creates a mock class instance of given class"

    ^ (self mockClassOf: aClass) new

    "Created: / 15-06-2014 / 23:44:04 / Jakub Nesveda <nesvejak@fit.cvut.cz>"
! !

!CustomMock methodsFor:'mock release'!

unmockAll

    mockedClasses do: [ :class | 
        class removeFromSystem
    ].

    mockedClasses removeAll

    "Created: / 15-06-2014 / 19:30:52 / Jakub Nesveda <nesvejak@fit.cvut.cz>"
! !

