"{ Package: 'jn:refactoring_custom' }"

TestCase subclass:#CustomSubclassResponsibilityCodeGeneratorTests
	instanceVariableNames:'codeGenerator mockSuperClass mockClass builder'
	classVariableNames:''
	poolDictionaries:''
	category:'Interface-Refactoring-Custom-Tests'
!

!CustomSubclassResponsibilityCodeGeneratorTests methodsFor:'initialization & release'!

setUp

    codeGenerator := CustomSubclassResponsibilityCodeGenerator new.
    codeGenerator dialog: CustomSilentDialog new.
    builder := CustomSourceCodeBuilder new.
    mockSuperClass := builder createClassImmediate: 'MockSuperClassForTestCase' superClassName: 'Object'.
    mockClass := builder createClassImmediate: 'MockClassForTestCase' superClassName: (mockSuperClass new className).

    "Created: / 14-04-2014 / 17:16:28 / Jakub Nesveda <nesvejak@fit.cvut.cz>"
    "Modified: / 15-06-2014 / 16:08:28 / Jakub Nesveda <nesvejak@fit.cvut.cz>"
!

tearDown

    builder redoChanges

    "Created: / 14-04-2014 / 17:16:56 / Jakub Nesveda <nesvejak@fit.cvut.cz>"
    "Modified: / 15-06-2014 / 16:11:55 / Jakub Nesveda <nesvejak@fit.cvut.cz>"
! !

!CustomSubclassResponsibilityCodeGeneratorTests methodsFor:'tests'!

test_none_subclass_responsibility
    | context |

    context := CustomSubContext new.
    context addSelectedClass: mockClass.

    builder createMethodImmediate: mockSuperClass protocol: 'instance-protocol' source: 'instanceMethod: aParam
    ^ self'.

    self assert: (mockClass includesSelector: #instanceMethod:) not. 
    self assert: (mockClass class includesSelector: #instanceMethod:) not.

    codeGenerator executeInContext: context.

    self assert: (mockClass includesSelector: #instanceMethod:) not.
    self assert: (mockClass class includesSelector: #instanceMethod:) not.

    "Created: / 13-05-2014 / 21:49:38 / Jakub Nesveda <nesvejak@fit.cvut.cz>"
    "Modified: / 15-06-2014 / 16:12:44 / Jakub Nesveda <nesvejak@fit.cvut.cz>"
!

test_subclass_responsibility_class_method
    | context |

    context := CustomSubContext new.
    context addSelectedClass: mockClass.

    builder createMethodImmediate: mockSuperClass class protocol: 'class-protocol' source: 'classMethod: aParam
    self subclassResponsibility'.

    self assert: (mockClass includesSelector: #classMethod:) not.
    self assert: (mockClass class includesSelector: #classMethod:) not.

    codeGenerator executeInContext: context.

    self assert: (mockClass includesSelector: #classMethod:) not.
    self assert: (mockClass class includesSelector: #classMethod:). 

    self assert: (mockClass class sourceCodeAt: #classMethod:) = ('classMethod:aParam
    self shouldImplement').

    "Created: / 15-04-2014 / 22:05:19 / Jakub Nesveda <nesvejak@fit.cvut.cz>"
    "Modified: / 15-06-2014 / 16:12:59 / Jakub Nesveda <nesvejak@fit.cvut.cz>"
!

test_subclass_responsibility_instance_method
    | context |

    context := CustomSubContext new.
    context addSelectedClass: mockClass.

    builder createMethodImmediate: mockSuperClass protocol: 'instance-protocol' source: 'instanceMethod: aParam
    self subclassResponsibility'.

    self assert: (mockClass includesSelector: #instanceMethod:) not. 
    self assert: (mockClass class includesSelector: #instanceMethod:) not.

    codeGenerator executeInContext: context.

    self assert: (mockClass includesSelector: #instanceMethod:).
    self assert: (mockClass class includesSelector: #instanceMethod:) not.

    self assert: (mockClass sourceCodeAt: #instanceMethod:) = ('instanceMethod:aParam
    self shouldImplement').

    "Created: / 14-04-2014 / 18:10:52 / Jakub Nesveda <nesvejak@fit.cvut.cz>"
    "Modified: / 15-06-2014 / 16:13:08 / Jakub Nesveda <nesvejak@fit.cvut.cz>"
! !

