"{ Package: 'jn:refactoring_custom' }"

CustomCodeGenerator subclass:#CustomTestCaseSetUpCodeGenerator
	instanceVariableNames:''
	classVariableNames:''
	poolDictionaries:''
	category:'Interface-Refactoring-Custom-Generators'
!

!CustomTestCaseSetUpCodeGenerator class methodsFor:'accessing-presentation'!

description
    ^ 'Generates initial #setUp for TestCases'

    "Modified: / 05-08-2014 / 13:37:59 / Jan Vrany <jan.vrany@fit.cvut.cz>"
!

group
    "Returns a collection strings describing a group to which
     receiver belongs. A groups may be nested hence the array of
     strings. For example for subgroup 'Accessors' in group 'Generators'
     this method should return #('Generators' 'Accessors')."

    "/ By default return an empty array which means the item will appear
    "/ in top-level group.
    ^ #('Testing')

    "Created: / 05-08-2014 / 14:14:20 / Jan Vrany <jan.vrany@fit.cvut.cz>"
!

label
    ^ 'TestCase setUp method'

    "Modified: / 05-08-2014 / 14:13:33 / Jan Vrany <jan.vrany@fit.cvut.cz>"
! !

!CustomTestCaseSetUpCodeGenerator class methodsFor:'queries'!

availableInContext:aCustomContext
    ^ aCustomContext selectedClasses anySatisfy: [:cls | cls isMetaclass not and:[ cls inheritsFrom: TestCase ] ]

    "Modified: / 05-08-2014 / 14:24:16 / Jan Vrany <jan.vrany@fit.cvut.cz>"
!

availableInPerspective:aCustomPerspective
    ^ aCustomPerspective isClassPerspective

    "Modified: / 05-08-2014 / 13:49:03 / Jan Vrany <jan.vrany@fit.cvut.cz>"
! !

!CustomTestCaseSetUpCodeGenerator methodsFor:'executing'!

buildForClass: class
    | source category superHasSetup current |

    current := class.
    superHasSetup := false.
    [ superHasSetup not and:[ current isNil not ] ] whileTrue:[
        superHasSetup := current includesSelector: #setUp.
        current := current superclass.
    ].
    (superHasSetup and:[ current name ~~ #TestCase ]) ifTrue:[ 
        source := 'setUp
    super setUp.

    "Add your own code here..."
'.
        category := (current compiledMethodAt: #setUp) category.
    ] ifFalse:[ 
        source := 'setUp
    "Add your own code here..."
'.
        category := (TestCase compiledMethodAt: #setUp) category.
    ].

    builder createMethod
        class: class;
        source: source;
        category: category.

    "Created: / 05-08-2014 / 14:17:17 / Jan Vrany <jan.vrany@fit.cvut.cz>"
!

buildInContext:aCustomContext
    aCustomContext selectedClasses do:[:cls |  
        (cls isMetaclass not and:[ cls inheritsFrom: TestCase ]) ifTrue:[ 
            self buildForClass: cls.
        ].
    ]

    "Modified: / 05-08-2014 / 14:24:34 / Jan Vrany <jan.vrany@fit.cvut.cz>"
! !

